BizonSuper = {};

function BizonSuper.prerequisitesPresent(specializations)
	return SpecializationUtil.hasSpecialization(Enterable, specializations) and SpecializationUtil.hasSpecialization(TurnOnVehicle, specializations) and SpecializationUtil.hasSpecialization(Combine, specializations)
end;

function BizonSuper.registerEventListeners(vehicleType)	
	SpecializationUtil.registerEventListener(vehicleType, "onLoad", BizonSuper)
	SpecializationUtil.registerEventListener(vehicleType, "onPostLoad", BizonSuper)
	SpecializationUtil.registerEventListener(vehicleType, "onDelete", BizonSuper)
	SpecializationUtil.registerEventListener(vehicleType, "onUpdate", BizonSuper)
	SpecializationUtil.registerEventListener(vehicleType, "onDraw", BizonSuper)
	SpecializationUtil.registerEventListener(vehicleType, "onReadStream", BizonSuper)
	SpecializationUtil.registerEventListener(vehicleType, "onWriteStream", BizonSuper)
	SpecializationUtil.registerEventListener(vehicleType, "onTurnedOn", BizonSuper)
end

function BizonSuper.registerOverwrittenFunctions(vehicleType)
    SpecializationUtil.registerOverwrittenFunction(vehicleType, "getIsAdditionalCharacterActive", BizonSuper.getIsAdditionalCharacterActive)
end

function BizonSuper.registerFunctions(vehicleType)
    SpecializationUtil.registerFunction(vehicleType, "setInsideCameraMode", BizonSuper.setInsideCameraMode)
end

function BizonSuper:onLoad(savegame)
    if self.loadDashboardsFromXML ~= nil then
        self:loadDashboardsFromXML(self.xmlFile, "vehicle.cylindered.dashboards", {valueTypeToLoad = "lowerImplement", valueObject = self, valueFunc = BizonSuper.lowerImplementDashboardState, minFunc = -1, maxFunc = 1, additionalAttributesFunc = Cylindered.movingToolDashboardAttributes, idleValue = 0})
	end
	
	self.onTurnedOnParticles = {}
	ParticleUtil.loadParticleSystem(self.xmlFile, self.onTurnedOnParticles, "vehicle.turnOnVehicle.onTurnedOnParticles.particle", self.components, false, nil, self.baseDirectory);
end
function BizonSuper:onPostLoad(savegame)
	local spec = self.spec_enterable
	if self.configurations["vehicleType"] == 3 or self.configurations["vehicleType"] == 4 then
		for i,camera in pairs(spec.cameras) do
			if camera.isInside then
				camera.useOutdoorSounds = false
			end
		end
	end
	
	self.cameraMode = true
end;

function BizonSuper:onDelete()
	if self.isClient then
		ParticleUtil.deleteParticleSystem(self.onTurnedOnParticles)
	end
end

function BizonSuper:onUpdate(dt)
	if self.isClient then 
		if self:getIsActive() and g_gui.currentGui == nil then	
			if self.spec_enterable.isEntered then
				if self.spec_enterable.activeCamera ~= nil then
					if self.spec_enterable.activeCamera.isInside then
						if Input.isKeyPressed(Input.KEY_lshift) then
							if not self:getIsAnimationPlaying("cameraAnimation") then
								if Input.isMouseButtonPressed(Input.MOUSE_BUTTON_WHEEL_UP) then
									if self:getAnimationTime("cameraAnimation") == 0 then
										self:setInsideCameraMode(false);
									end
								elseif Input.isMouseButtonPressed(Input.MOUSE_BUTTON_WHEEL_DOWN) then
									if self:getAnimationTime("cameraAnimation") == 1 then
										self:setInsideCameraMode(true);
									end
								end;
							end
						end;
					end
				end
			end
		end
	end
	
	if self.particlesStarted then
		if not g_soundManager:getIsSamplePlaying(self.spec_combine.samples.start) then
			if self.isClient then
				ParticleUtil.setEmittingState(self.onTurnedOnParticles, false)
			end
			self.particlesStarted = nil
		end
	end
end

function BizonSuper:onDraw()
	if self.spec_enterable.activeCamera ~= nil then
		if self.spec_enterable.activeCamera.isInside then
			g_currentMission:addExtraPrintText(g_i18n:getText("input_cameraInfo"));
		end
	end
end

function BizonSuper:lowerImplementDashboardState()	
	if self.spec_attacherJoints.attachedImplements[1] ~= nil then
		self.implementMove = 1 - self.spec_attacherJoints.attacherJoints[1].moveAlpha 
		if self.spec_attacherJoints.attacherJoints[1].moveAlpha < 0.8 then
			self.implementMove = self.spec_attacherJoints.attacherJoints[1].moveAlpha 
		end
		if self.spec_attacherJoints.attachedImplements[1].object:getIsLowered() == true then
			return self.implementMove * 15
		elseif self.spec_attacherJoints.attachedImplements[1].object:getIsLowered() == false then
			return -self.implementMove * 15
		end
	end
	return 0
end

function BizonSuper:getIsAdditionalCharacterActive()
	return not self.cameraMode
end;

function BizonSuper:setInsideCameraMode(cameraMode, noEventSend)
	SetInsideCameraEvent.sendEvent(self, cameraMode, noEventSend);
	self.cameraMode = cameraMode;
	if cameraMode then
		if self.playAnimation ~= nil then
			self:playAnimation("cameraAnimation", -1, nil, true);
		end;
	else
		if self.playAnimation ~= nil then
			self:playAnimation("cameraAnimation", 1, nil, true);
		end;
	end;
end;

function BizonSuper:onReadStream(streamId, connection)
	self:setInsideCameraMode(streamReadBool(streamId), true);
end;

function BizonSuper:onWriteStream(streamId, connection)
	streamWriteBool(streamId, self.cameraMode);
end;

function BizonSuper:onTurnedOn()
	if self.isClient then
		ParticleUtil.setEmittingState(self.onTurnedOnParticles, true)
		self.particlesStarted = true
    end
end

SetInsideCameraEvent = {};
SetInsideCameraEvent_mt = Class(SetInsideCameraEvent, Event);

InitEventClass(SetInsideCameraEvent, "SetInsideCameraEvent");

function SetInsideCameraEvent:emptyNew()
    local self = Event:new(SetInsideCameraEvent_mt);
    self.className="SetInsideCameraEvent";
    return self;
end;

function SetInsideCameraEvent:new(vehicle, isCameraMode)
    local self = SetInsideCameraEvent:emptyNew()
    self.vehicle = vehicle;
	self.isCameraMode = isCameraMode;
    return self;
end;

function SetInsideCameraEvent:readStream(streamId, connection)
    self.vehicle = NetworkUtil.readNodeObject(streamId)
	self.isCameraMode = streamReadBool(streamId);
    self:run(connection);
end;

function SetInsideCameraEvent:writeStream(streamId, connection)
    NetworkUtil.writeNodeObject(streamId, self.vehicle)
	streamWriteBool(streamId, self.isCameraMode);
end;

function SetInsideCameraEvent:run(connection)   
	self.vehicle:setInsideCameraMode(self.isCameraMode, true);
    if not connection:getIsServer() then
        g_server:broadcastEvent(SetInsideCameraEvent:new(self.vehicle, self.isCameraMode), nil, connection, self.vehicle);
    end;
end;

function SetInsideCameraEvent.sendEvent(vehicle, isCameraMode, noEventSend)
	if noEventSend == nil or noEventSend == false then
		if g_server ~= nil then
			g_server:broadcastEvent(SetInsideCameraEvent:new(vehicle, isCameraMode), nil, nil, vehicle);
		else
			g_client:getServerConnection():sendEvent(SetInsideCameraEvent:new(vehicle, isCameraMode));
		end;
	end;
end;