ChopperConfigurations = {};

function ChopperConfigurations.initSpecialization()
	if g_configurationManager.configurations["chopper"] == nil then
		g_configurationManager:addConfigurationType("chopper", g_i18n:getText("configuration_choppa"), nil, nil, nil, nil, ConfigurationUtil.SELECTOR_MULTIOPTION)
	end
end;

function ChopperConfigurations.prerequisitesPresent(specializations)
	return SpecializationUtil.hasSpecialization(Combine, specializations)
end;

function ChopperConfigurations.registerEventListeners(vehicleType)	
	SpecializationUtil.registerEventListener(vehicleType, "onLoad", ChopperConfigurations)
	SpecializationUtil.registerEventListener(vehicleType, "onPostLoad", ChopperConfigurations)
end

function ChopperConfigurations:onLoad(savegame)
	ObjectChangeUtil.updateObjectChanges(self.xmlFile, "vehicle.chopperConfigurations.chopperConfiguration", self.configurations["chopper"], self.components, self);
end;

function ChopperConfigurations:onPostLoad(savegame)
	ChopperConfigurations.loadChopperConfigurations(self, self.xmlFile, self.configurations["chopper"], self.spec_combine);
end;

function ChopperConfigurations:loadChopperConfigurations(xmlFile, configId, entry)
	local key = string.format("vehicle.chopperConfigurations.chopperConfiguration(%d)", configId-1)

    entry.swath = {}
    entry.swath.isAvailable = Utils.getNoNil(getXMLBool(xmlFile, key .. ".swath#available"), false)
    local isDefaultActive = Utils.getNoNil(getXMLBool(xmlFile, key .. ".swath#isDefaultActive"), entry.swath.isAvailable)
    if entry.swath.isAvailable then
        entry.swath.workAreaIndex = getXMLInt(xmlFile, key .. ".swath#workAreaIndex")
        if entry.swath.workAreaIndex == nil then
            entry.swath.isAvailable = false
            g_logManager:xmlWarning(self.configFileName, "Missing 'swath#workAreaIndex' for combine swath function!")
        end
        entry.warningTime = 0
    end

    entry.chopper = {}
    entry.chopper.isAvailable = Utils.getNoNil(getXMLBool(xmlFile, key .. ".chopper#available"), false)
    if entry.chopper.isAvailable then
        entry.chopper.workAreaIndex = getXMLInt(xmlFile, key .. ".chopper#workAreaIndex")
        if entry.chopper.workAreaIndex == nil then
            entry.chopper.isAvailable = false
            g_logManager:xmlWarning(self.configFileName, "Missing 'chopper#workAreaIndex' for combine chopper function!")
        end
        entry.chopper.animName = getXMLString(xmlFile, key .. ".chopper#animName")
        entry.chopper.animSpeedScale = getXMLFloat(xmlFile, key .. ".chopper#animSpeedScale")
    end
    entry.isSwathActive = isDefaultActive
	
    entry.processing = {}
    local toggleTime = getXMLFloat(xmlFile, key .. ".processing#toggleTime")
    if toggleTime == nil and entry.chopper.animName ~= nil then
        toggleTime = self:getAnimationDurection(entry.chopper.animName)
        if toggleTime ~= nil then
            toggleTime = toggleTime / 1000
        end
    end
	
    entry.processing.toggleTime = Utils.getNoNil(toggleTime, 0) * 1000
	
    local inputBuffer = {}
    local slotDuration = 300
    local slotCount = MathUtil.clamp(math.ceil(entry.processing.toggleTime / slotDuration), 2, 20)
    inputBuffer.slotCount = slotCount
    inputBuffer.slotDuration = math.ceil( entry.processing.toggleTime / inputBuffer.slotCount )
    inputBuffer.fillIndex = 1
    inputBuffer.dropIndex = inputBuffer.fillIndex + 1
    inputBuffer.slotTimer = inputBuffer.slotDuration
    inputBuffer.activeTimeout = inputBuffer.slotDuration * (inputBuffer.slotCount + 2)
    inputBuffer.activeTimer = inputBuffer.activeTimeout
    inputBuffer.buffer = {}
    for i=1, inputBuffer.slotCount do
        table.insert(inputBuffer.buffer, {realArea = 0, area = 0, liters = 0, inputLiters = 0, strawRatio = 0} )
    end
    entry.processing.inputBuffer = inputBuffer
end;