PipeExtension = {};

function PipeExtension.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(Pipe, specializations) and SpecializationUtil.hasSpecialization(AIVehicle, specializations) and SpecializationUtil.hasSpecialization(Dischargeable, specializations)
end

function PipeExtension.registerEventListeners(vehicleType)
    SpecializationUtil.registerEventListener(vehicleType, "onLoad", PipeExtension)
	SpecializationUtil.registerEventListener(vehicleType, "onDelete", PipeExtension)
	SpecializationUtil.registerEventListener(vehicleType, "onUpdate", PipeExtension)
	SpecializationUtil.registerEventListener(vehicleType, "onRegisterActionEvents", PipeExtension)
	SpecializationUtil.registerEventListener(vehicleType, "onDischargeStateChanged", PipeExtension)
end

function PipeExtension.registerFunctions(vehicleType)
	SpecializationUtil.registerFunction(vehicleType, "playerTriggerCheck", PipeExtension.playerTriggerCheck)
end

function PipeExtension.registerOverwrittenFunctions(vehicleType)
    SpecializationUtil.registerOverwrittenFunction(vehicleType, "updateAIDriveStrategies", PipeExtension.updateAIDriveStrategies)
	SpecializationUtil.registerOverwrittenFunction(vehicleType, "getCanToggleDischargeToObject", PipeExtension.getCanToggleDischargeToObject)
	SpecializationUtil.registerOverwrittenFunction(vehicleType, "handleDischargeRaycast", PipeExtension.handleDischargeRaycast)
end

function PipeExtension:onLoad(savegame)
	self.playerTrigger = I3DUtil.indexToObject(self.components, getXMLString(self.xmlFile, "vehicle.pipe#triggerNode"));
	addTrigger(self.playerTrigger, "playerTriggerCheck", self); 
	
	self.playerInTrigger = false
	
    self.pipeUnloadAnimation = {}
    self.pipeUnloadAnimation.name = getXMLString(self.xmlFile, "vehicle.pipe.unloadAnimation#name")
    self.pipeUnloadAnimation.speedScale = Utils.getNoNil(getXMLFloat(self.xmlFile, "vehicle.pipe.unloadAnimation#speedScale"), 1)
end

function PipeExtension:onDelete()
	removeTrigger(self.playerTrigger)
    g_inputBinding:beginActionEventsModification(Player.INPUT_CONTEXT_NAME)
    g_inputBinding:removeActionEventsByTarget(self)
    g_inputBinding:endActionEventsModification()
end

function PipeExtension:onUpdate(dt, isActiveForInput, isActiveForInputIgnoreSelection, isSelected)
	if g_gui.currentGuiName == "ShopConfigScreen" then
		if g_currentMission.shopController.buyItemFilename ~= nil then
			if g_currentMission.shopController.buyItemFilename == self.configFileName then
				if g_currentMission.shopController.isBuying then
					if g_currentMission.shopController.buyVehicleNow > 0 then
						self.loadInfoDialog = true
					end
					if self.loadInfoDialog then
						if self.lockExistingDialog == nil then
							g_gui:showInfoDialog({text=g_i18n:getText("pipeAdditionalDialog"), dialogType=DialogElement.TYPE_BENCHMARK})
							if g_currentMission.shopController.buyVehicleNow == 0 then
								self.lockExistingDialog = true
							end
						end
					end
				end
			end
		end
	end

	if self.playerInTrigger then
		PipeExtension.updateActionEventText(self)
		self:raiseActive()
	end;
end

function PipeExtension:onRegisterActionEvents(isActiveForInput, isActiveForInputIgnoreSelection)
    if self.isClient then
        local spec = self.spec_pipe
        self:clearActionEventsTable(spec.actionEvents)
        if isActiveForInputIgnoreSelection and spec.hasMovablePipe then
			if spec.actionEvents[InputAction.TOGGLE_PIPE] ~= nil then
				spec.actionEvents[InputAction.TOGGLE_PIPE] = nil
			end
        end
    end
end

function PipeExtension:playerTriggerCheck(triggerId, otherId, onEnter, onLeave, onStay)
    if onEnter and g_currentMission.controlPlayer and g_currentMission.player ~= nil and otherId == g_currentMission.player.rootNode then
		self.pipeActionEvents = {};
		self:clearActionEventsTable(self.pipeActionEvents)
		_, self.actionEventId = self:addActionEvent(self.pipeActionEvents, InputAction.TOGGLE_PIPE, self, Pipe.actionEventTogglePipe, false, true, false, true, nil);
		PipeExtension.updateActionEventText(self)
		self:raiseActive()
		self.playerInTrigger = true
    elseif onLeave and g_currentMission.player ~= nil and otherId == g_currentMission.player.rootNode then
		self:removeActionEvent(self.pipeActionEvents, InputAction.TOGGLE_PIPE);
		self.playerInTrigger = false
    end;
end;

function PipeExtension.updateActionEventText(self)
    local spec = self.spec_pipe
    local actionEvent = self.pipeActionEvents[InputAction.TOGGLE_PIPE]
    if actionEvent ~= nil then
        local showAction = false
        if spec.targetState == spec.numStates then
            if self:getIsPipeStateChangeAllowed(1) then
                g_inputBinding:setActionEventText(actionEvent.actionEventId, g_i18n:getText("action_pipeIn"))
                showAction = true
            end
        else
            local nextState = spec.targetState + 1
            if self:getIsPipeStateChangeAllowed(nextState) then
                local pipeStateName = ""
                if spec.numUnloadingStates > 1 and spec.numUnloadingStates ~= spec.numStates then
                    pipeStateName = string.format(" [%d]", nextState - 1)
                    local dischargeNodeIndex = self:getPipeDischargeNodeIndex(nextState)
                    local dischargeNode = self:getDischargeNodeByIndex(dischargeNodeIndex)
                    local fillTypeIndex = self:getFillUnitFillType(dischargeNode.fillUnitIndex)
                    if fillTypeIndex ~= FillType.UNKNOWN then
                        local fillType = g_fillTypeManager:getFillTypeByIndex(fillTypeIndex)
                        if fillType ~= nil then
                            pipeStateName = string.format(" [%d, %s]", nextState - 1, fillType.title)
                        end
                    end
                end
                g_inputBinding:setActionEventText(actionEvent.actionEventId, string.format(g_i18n:getText("action_pipeOut"), pipeStateName))
                showAction = true
            end
        end
        g_inputBinding:setActionEventActive(actionEvent.actionEventId, showAction)
    end
end

function PipeExtension:updateAIDriveStrategies(superfunc)
    local spec = self.spec_aiVehicle
    if #spec.aiImplementList > 0 then
        if spec.driveStrategies ~= nil and #spec.driveStrategies > 0 then
            for i=#spec.driveStrategies,1,-1 do
                spec.driveStrategies[i]:delete()
                table.remove(spec.driveStrategies, i)
            end
            spec.driveStrategies = {}
        end
        local foundCombine = false
        local foundBaler = false
        for _,implement in pairs(spec.aiImplementList) do
            if SpecializationUtil.hasSpecialization(Combine, implement.object.specializations) then
                foundCombine = true
            end
            if SpecializationUtil.hasSpecialization(Baler, implement.object.specializations) then
                foundBaler = true
            end
        end
        foundCombine = foundCombine or SpecializationUtil.hasSpecialization(Combine, spec.specializations)
        if foundCombine then
            local driveStrategyCombine = AIDriveStrategyCombine:new()
            driveStrategyCombine:setAIVehicle(self)
            table.insert(spec.driveStrategies, driveStrategyCombine)
			for i=#spec.driveStrategies,1,-1 do
				spec.driveStrategies[i].getDriveData = PipeExtension.getDriveData
			end
        end
        foundBaler = foundBaler or SpecializationUtil.hasSpecialization(Baler, spec.specializations)
        if foundBaler then
            local driveStrategyCombine = AIDriveStrategyBaler:new()
            driveStrategyCombine:setAIVehicle(self)
            table.insert(spec.driveStrategies, driveStrategyCombine)
        end
        local driveStrategyCollision = AIDriveStrategyCollision:new()
        local driveStrategyStraight = AIDriveStrategyStraight:new()
        driveStrategyCollision:setAIVehicle(self)
        driveStrategyStraight:setAIVehicle(self)
        table.insert(spec.driveStrategies, driveStrategyCollision)
        table.insert(spec.driveStrategies, driveStrategyStraight)
    end
end

function PipeExtension:getDriveData(dt, vX,vY,vZ)
    local isTurning = self.vehicle:getRootVehicle():getAIIsTurning()
    local allowedToDrive = true
    local waitForStraw = false
    local maxSpeed = math.huge
    for _, combine in pairs(self.combines) do
        if not combine:getIsThreshingAllowed() then
            self.vehicle:stopAIVehicle(AIVehicle.STOP_REASON_REGULAR)
            self:debugPrint("Stopping AIVehicle - combine not allowed to thresh")
            return nil, nil, nil, nil, nil
        end
        if combine.spec_pipe ~= nil then
            local fillLevel = 0
            local capacity = 0
            local trailerInTrigger = false
            local invalidTrailerInTrigger = false
            local dischargeNode = combine:getCurrentDischargeNode()
            if dischargeNode ~= nil then
                fillLevel = combine:getFillUnitFillLevel(dischargeNode.fillUnitIndex)
                capacity = combine:getFillUnitCapacity(dischargeNode.fillUnitIndex)
            end
            local trailer = NetworkUtil.getObject(combine.spec_pipe.nearestObjectInTriggers.objectId)
            local trailerFillUnitIndex = combine.spec_pipe.nearestObjectInTriggers.fillUnitIndex
            if trailer ~= nil then
                trailerInTrigger = true
            end
            if combine.spec_pipe.nearestObjectInTriggerIgnoreFillLevel then
                invalidTrailerInTrigger = true
            end
            local currentPipeTargetState = combine.spec_pipe.targetState
            if capacity == math.huge then
                -- forage harvesters
                if currentPipeTargetState ~= 2 then
                    combine:setPipeState(2)
                end
                if not isTurning then
                    if trailerInTrigger then
                        local fillType = combine:getDischargeFillType(dischargeNode)
                        if fillType == FillType.UNKNOWN then
                            -- if nothing is in combine fillUnit we just check if we're targetting the trailer with the trailers first fill type or the current fill type if something is loaded
                            fillType = trailer:getFillUnitFillType(trailerFillUnitIndex)
                            if fillType == FillType.UNKNOWN then
                                fillType = trailer:getFillUnitFirstSupportedFillType(trailerFillUnitIndex)
                            end
                            combine:setForcedFillTypeIndex(fillType)
                        else
                            -- otherwise we check if the fill type of the combine is supported on the trailer
                            combine:setForcedFillTypeIndex(nil)
                        end
                    end
                    local targetObject, _ = combine:getDischargeTargetObject(dischargeNode)
                    allowedToDrive = trailerInTrigger and targetObject ~= nil
                    if not trailerInTrigger then
                        self.vehicle:addAIDebugText("COMBINE -> Waiting for trailer enter the trigger")
                    elseif trailerInTrigger and targetObject == nil then
                        self.vehicle:addAIDebugText("COMBINE -> Waiting for pipe hitting the trailer")
                    end
                end
            else
                -- combine harvesters
                local pipeState = currentPipeTargetState
                if fillLevel > (0.8*capacity) then
                    if not self.beaconLightsActive then
                        self.vehicle:setAIMapHotspotBlinking(true)
                        self.vehicle:setBeaconLightsVisibility(true)
                        self.beaconLightsActive = true
                    end
                    if not self.notificationGrainTankWarningShown then
                        --g_currentMission:addIngameNotification(FSBaseMission.INGAME_NOTIFICATION_CRITICAL, string.format(g_i18n:getText("ingameNotification_aiVehicleReasonGrainTankIsNearlyFull"), self.vehicle:getCurrentHelper().name) )
						g_currentMission:addIngameNotification(FSBaseMission.INGAME_NOTIFICATION_CRITICAL, string.format("Zbiornik jest prawie pełen!", self.vehicle:getCurrentHelper().name) )
                        self.notificationGrainTankWarningShown = true
                    end
                else
                    if self.beaconLightsActive then
                        self.vehicle:setAIMapHotspotBlinking(false)
                        self.vehicle:setBeaconLightsVisibility(false)
                        self.beaconLightsActive = false
                    end
                    self.notificationGrainTankWarningShown = false
                end
                if fillLevel == capacity then
                    pipeState = 2
                    self.wasCompletelyFull = true
                    if self.notificationFullGrainTankShown ~= true then
                        --g_currentMission:addIngameNotification(FSBaseMission.INGAME_NOTIFICATION_CRITICAL, string.format(g_i18n:getText(AIVehicle.REASON_TEXT_MAPPING[AIVehicle.STOP_REASON_GRAINTANK_IS_FULL]), self.vehicle:getCurrentHelper().name) )
						g_currentMission:addIngameNotification(FSBaseMission.INGAME_NOTIFICATION_CRITICAL, string.format("Zbiornik jest pełen!", self.vehicle:getCurrentHelper().name) )
                        self.notificationFullGrainTankShown = true
                    end
                else
                    self.notificationFullGrainTankShown = false
                end
                if trailerInTrigger then
                    pipeState = 2
                end
                if not trailerInTrigger then
                    if fillLevel < capacity * 0.8 then
                        self.wasCompletelyFull = false
                        if not combine:getIsTurnedOn() then
                            combine:setIsTurnedOn(true)
                        end
                    end
                end
                if (not trailerInTrigger and not invalidTrailerInTrigger) and fillLevel < capacity then
                    pipeState = 1
                end
                if fillLevel < 0.1 then
                    if not combine.spec_pipe.aiFoldedPipeUsesTrailerSpace then
                        if not trailerInTrigger and not invalidTrailerInTrigger then
                            pipeState = 1
                        end
                        if not combine:getIsTurnedOn() then
                            combine:setIsTurnedOn(true)
                        end
                    end
                    self.wasCompletelyFull = false
                end
				if pipeState == 2 then
					if fillLevel > 0.1 and trailerInTrigger then
						if combine:getDischargeState() == Dischargeable.DISCHARGE_STATE_OFF then
							combine:setDischargeState(Dischargeable.DISCHARGE_STATE_OBJECT, true)
						end
					elseif fillLevel <= 0.1 or not trailerInTrigger then
						if combine:getDischargeState() == Dischargeable.DISCHARGE_STATE_OBJECT then
							combine:setDischargeState(Dischargeable.DISCHARGE_STATE_OFF, true)
						end
					end
				end
                allowedToDrive = fillLevel < capacity
                if pipeState == 2 and self.wasCompletelyFull then
                    allowedToDrive = false
                    self.vehicle:addAIDebugText("COMBINE -> Waiting for trailer to unload")
                end
                if isTurning and trailerInTrigger then
                    if combine:getCanDischargeToObject(dischargeNode) then
                        allowedToDrive = fillLevel == 0
                        if not allowedToDrive then
                            self.vehicle:addAIDebugText("COMBINE -> Unload to trailer on headland")
                        end
                    end
                end
                local freeFillLevel = capacity - fillLevel
                if freeFillLevel < self.slowDownFillLevel then
                    -- we want to drive at least 2 km/h to avoid combine stops too early
                    maxSpeed = 2 + (freeFillLevel / self.slowDownFillLevel) * self.slowDownStartSpeed
                    self.vehicle:addAIDebugText(string.format("COMBINE -> Slow down because nearly full: %.2f", maxSpeed))
                end
            end
            if not trailerInTrigger then
                if combine.spec_combine.isSwathActive then
                    if combine.spec_combine.strawPSenabled then
                        waitForStraw = true
                    end
                end
            end
        end
    end
    if isTurning and waitForStraw then
        self.vehicle:addAIDebugText("COMBINE -> Waiting for straw to drop")
        local h = getTerrainHeightAtWorldPos(g_currentMission.terrainRootNode, self.vehicle.aiDriveTarget[1],0,self.vehicle.aiDriveTarget[2])
        local x,_,z = worldToLocal(self.vehicle:getAIVehicleDirectionNode(), self.vehicle.aiDriveTarget[1],h,self.vehicle.aiDriveTarget[2])
        local dist = MathUtil.vector2Length(vX-x, vZ-z)
        return x, z, false, 10, dist
    else
        if not allowedToDrive then
            return 0, 1, true, 0, math.huge
        else
            return nil, nil, nil, maxSpeed, nil
        end
    end
end

function PipeExtension:onDischargeStateChanged(state)
    if self.isClient then
        local spec = self.spec_pipe
        local dischargeNode = self:getCurrentDischargeNode()
        local dischargeNodeIndex = nil
        if dischargeNode ~= nil then
            dischargeNodeIndex = dischargeNode.index
        end
        if dischargeNodeIndex == spec.dischargeNodeIndex then
            if state == Dischargeable.DISCHARGE_STATE_OFF then
                g_animationManager:stopAnimations(spec.animationNodes)
				self:playAnimation(self.pipeUnloadAnimation.name, -self.pipeUnloadAnimation.speedScale, nil, true)
            else
                g_animationManager:startAnimations(spec.animationNodes)
                g_animationManager:setFillType(spec.animationNodes, self:getFillUnitLastValidFillType(dischargeNode.fillUnitIndex))
				self:playAnimation(self.pipeUnloadAnimation.name, self.pipeUnloadAnimation.speedScale, nil, true)
            end
        end
    end
end

function PipeExtension:getCanToggleDischargeToObject(superFunc)
    return true
end

function PipeExtension:handleDischargeRaycast(superfunc, dischargeNode, object, shape, distance, illUnitIndex, hitTerrain)
    local spec = self.spec_dischargeable
    -- stop tipping if discharge to object is active but object is not hit anymore
    if object == nil and spec.currentDischargeState == Dischargeable.DISCHARGE_STATE_OBJECT then
        self:setDischargeState(Dischargeable.DISCHARGE_STATE_OFF)
    end
end